/*
 * Authors: Timo Hoenig <thoenig@nouse.net>
 *          Daniel Silverstone <daniel.silverstone@canonical.com>
 * Copyright (c) 2003, 2004 Timo Hoenig <thoenig@nouse.net>
 *               2005 Daniel Silverstone <daniel.silverstone@canonical.com>
 *
 *                          All rights reserved
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

/*
 * Defines
 *
 */

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <stdio.h>

/*
 * Declarations
 *
 */

void acpi_mainloop(void);
int keys_over_acpi(void);

/*
 * Includes
 *
 */

#include "fnfx.h"

/*
 * External functions
 *
 */

extern void debug(const char *fmt, ...) __attribute__ ((format(printf, 1, 2)));
extern void fatal(const char *fmt, ...) __attribute__ ((format(printf, 1, 2)));
extern int lookup(int);
extern int do_action(int);
extern void clean_config(void);
extern void signal_fnfx();
extern void wait_fnfx();
extern int notify_client();

/*
 * External variables
 *
 */

extern t_daemon_cfg *cfg;

/*
 * Functions
 *
 */

/*
 * chk_fnkey()
 *
 * Scan /proc/acpi/toshiba/keys for Fn-key events.
 *
 * If a new key event is found return 1, otherwise 0.
 *
 */

static void do_fnkey()
{
  int fnkey_pos = -1;
  
  fnkey_pos = lookup(cfg->acpi.hotkey);
  cfg->acpi.fnkey_descr = cfg->keymap.fnkey[fnkey_pos].descr;
  cfg->acpi.fnkey_pos = fnkey_pos;
    
  if (fnkey_pos != -1) {
    if (notify_client()) {
      if (cfg->debug_flag)
        debug("chk_fnkey() - %s (0x%x). Executing command: %s.",
              cfg->keymap.fnkey[fnkey_pos].descr,
              cfg->acpi.hotkey,
              ((cfg->keymap.fnkey[fnkey_pos].
                state) & FNKEY_ENABLED) ? cfg->keymap.
              fnkey[fnkey_pos].command : "None");
      
      if (cfg->keymap.fnkey[fnkey_pos].state & FNKEY_ENABLED) {
        cfg->keymap.fnkey[fnkey_pos].state &= ~FNKEY_CLIENT;
        do_action(fnkey_pos);
      }
    }
    else {
      if (strlen(cfg->keymap.fnkey[fnkey_pos].client_command)) {
        if (!
            (cfg->keymap.fnkey[fnkey_pos].
             state & FNKEY_EXTERNAL)) {
          if (cfg->debug_flag)
            debug
              ("chk_fnkey() - %s (0x%x). Executing command for client: %s.",
               cfg->keymap.fnkey[fnkey_pos].descr,
               cfg->acpi.hotkey,
               ((cfg->keymap.fnkey[fnkey_pos].
                 state) & FNKEY_ENABLED) ? cfg->keymap.
               fnkey[fnkey_pos].client_command : "None");
          
          cfg->keymap.fnkey[fnkey_pos].state |= FNKEY_CLIENT;
          do_action(fnkey_pos);
        }
      }
    }
  }
}


/*
 * keys_over_acpi()
 *
 * return 1 if hotkeys are over acpi.
 * If hotkeys *are* over acpi; then we enter a different main loop
 */

int keys_over_acpi(void)
{
  FILE *f = fopen(ACPI_KEYS, "r+");
  int ret = 0, ignore1, ignore2;
  debug("Attempting to determine if hotkeys are over acpi...\n");
  fscanf(f, "hotkey_ready: %d\nhotkey: 0x%4x\nhotkeys_via_acpi: %d",
         &ignore1, &ignore2, &ret);
  fclose(f);
  debug("Result of scan is %d\n", ret);
  return ret;
}


/*
 * acpi_mainloop()
 *
 * The main loop mentioned above
 */
void acpi_mainloop(void)
{
  int acpi_socket;
  struct sockaddr_un addr;
  /* Connect a socket */
  if( (acpi_socket = socket(PF_FILE, SOCK_STREAM, 0)) == -1 ) {
    return; /* Failure to initialise */
  }
  addr.sun_family = AF_FILE;
  strcpy(addr.sun_path, "/var/run/acpid.socket");
  if( connect(acpi_socket, (struct sockaddr*)&addr, sizeof(addr)) == -1 ) {
    close(acpi_socket);
    return; /* Failure to initialise */
  }
  /* Since fnfxd is naturally blocking; let's read an event at a time */
  for(;;) {
    char buf[1024];
    int r = 0;
    buf[r] = 0;
    while( r < 1023 ) {
      if( read(acpi_socket, buf+r, 1) != 1 ) {
        /* Something remarkably odd happened; close the socket and return */
        close(acpi_socket);
        return; /* Failure while running */
      }
      if( buf[r] == '\n' ) break;
      r++;
    }
    buf[r] = '\0';
    /* One event in buf, process it */
    if( buf[0] == 'h' &&
        buf[1] == 'k' &&
        buf[2] == 'e' &&
        buf[3] == 'y' &&
        buf[4] == ' ' &&
        buf[5] == 'V' &&
        buf[6] == 'A' &&
        buf[7] == 'L' &&
        (buf[8] == 'D' || buf[8] == 'Z') &&
        buf[9] == ' ' ) {
      /* It smells like a toshiba hotkey event... */
      int hkey = 0;
      sscanf(buf+19, "%x", &hkey);
      if( buf[17] == '1' ) hkey |= 0x80; /* keydown event */
      cfg->acpi.hotkey = hkey;
      do_fnkey();
    }
  }
}
